# coding: utf-8
from contextlib import contextmanager
from functools  import wraps
import six
import sys


__all__ = ("contextdecorator", "nested", "RequestFactory")


def contextdecorator(yielder):
    """
    Given a function that returns a contextmanager style generator, return an
    object that can be used as a context for an Attest collection, or used to
    decorate an individual test.

    Usage::

        # Define the context/decorator
        @contextdecorator
        def database(port=5432):
            try:
                db.open(port)
                yield
            finally:
                db.close()

        # Can decorate functions...
        @database
        def foo():
            ...

        # and supports keyword arguments...
        @database(port=6543)
        def foo():
            ...

        # but can still be used with contextlib.contextmanager
        with contextmanager(database)():
            ...
        with contextmanager(database(port=6543))():
            ...

        # which means it can be used as a test collection context
        suite = Tests()
        suite.context(database)
        suite.context(database(port=6532))

    .. note::

        *yielder* arguments must be optional keywords.

    """
    @wraps(yielder)
    def thing(func=None, **kwargs):
        def wrap(func):
            # wrap func using yielder as a context manager
            @wraps(func)
            def wrapped():
                with contextmanager(yielder)(**kwargs):
                    return func()
            return wrapped

        if func:
            # @thing
            # def foo():
            #     ...
            return wrap(func)

        elif kwargs:
            @wraps(yielder)
            def wrapped(func=None):
                if func:
                    # @thing(foo='bar')
                    # def foo():
                    #     ...
                    return wrap(func)
                else:
                    # contextlib.contextmanager(thing(foo='bar'))
                    return yielder(**kwargs)
            return wrapped

        else:
            # contextlib.contextmanager(thing)
            return yielder()
    return thing


@contextmanager
def nested(*managers):
    exc = None, None, None
    args = []
    exits = []
    try:
        for manager in managers:
            args.append(manager.__enter__())
            exits.append(manager.__exit__)
        yield args
    except:
        exc = sys.exc_info()
    finally:
        for exit in reversed(exits):
            try:
                if exit(*exc):
                    exc = None, None, None
            except:
                exc = sys.exc_info()
        if exc != (None, None, None):
            six.reraise(*exc)


try:
    from django.test.client import RequestFactory
except ImportError:
    # Provide our own version, copied from Django 1.4
    from django.test.client import (
            BOUNDARY, CONTENT_TYPE_RE, encode_multipart, FakePayload,
            MULTIPART_CONTENT,  StringIO, SimpleCookie, smart_str, WSGIRequest,
            settings, urlencode, urllib, urlparse)

    class RequestFactory(object):
        def __init__(self, **defaults):
            self.defaults = defaults
            self.cookies = SimpleCookie()
            self.errors = StringIO()

        def _base_environ(self, **request):
            environ = {
                'HTTP_COOKIE':       self.cookies.output(header='', sep='; '),
                'PATH_INFO':         '/',
                'REMOTE_ADDR':       '127.0.0.1',
                'REQUEST_METHOD':    'GET',
                'SCRIPT_NAME':       '',
                'SERVER_NAME':       'testserver',
                'SERVER_PORT':       '80',
                'SERVER_PROTOCOL':   'HTTP/1.1',
                'wsgi.version':      (1, 0),
                'wsgi.url_scheme':   'http',
                'wsgi.input':        FakePayload(''),
                'wsgi.errors':       self.errors,
                'wsgi.multiprocess': True,
                'wsgi.multithread':  False,
                'wsgi.run_once':     False,
            }
            environ.update(self.defaults)
            environ.update(request)
            return environ

        def request(self, **request):
            return WSGIRequest(self._base_environ(**request))

        def _encode_data(self, data, content_type, ):
            if content_type is MULTIPART_CONTENT:
                return encode_multipart(BOUNDARY, data)
            else:
                # Encode the content so that the byte representation is correct.
                match = CONTENT_TYPE_RE.match(content_type)
                if match:
                    charset = match.group(1)
                else:
                    charset = settings.DEFAULT_CHARSET
                return smart_str(data, encoding=charset)

        def _get_path(self, parsed):
            # If there are parameters, add them
            if parsed[3]:
                return urllib.unquote(parsed[2] + ";" + parsed[3])
            else:
                return urllib.unquote(parsed[2])

        def get(self, path, data={}, **extra):
            parsed = urlparse(path)
            r = {
                'CONTENT_TYPE':    'text/html; charset=utf-8',
                'PATH_INFO':       self._get_path(parsed),
                'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'GET',
            }
            r.update(extra)
            return self.request(**r)

        def post(self, path, data={}, content_type=MULTIPART_CONTENT,
                 **extra):
            "Construct a POST request."

            post_data = self._encode_data(data, content_type)

            parsed = urlparse(path)
            r = {
                'CONTENT_LENGTH': len(post_data),
                'CONTENT_TYPE':   content_type,
                'PATH_INFO':      self._get_path(parsed),
                'QUERY_STRING':   parsed[4],
                'REQUEST_METHOD': 'POST',
                'wsgi.input':     FakePayload(post_data),
            }
            r.update(extra)
            return self.request(**r)

        def head(self, path, data={}, **extra):
            "Construct a HEAD request."

            parsed = urlparse(path)
            r = {
                'CONTENT_TYPE':    'text/html; charset=utf-8',
                'PATH_INFO':       self._get_path(parsed),
                'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'HEAD',
            }
            r.update(extra)
            return self.request(**r)

        def options(self, path, data={}, **extra):
            "Constrict an OPTIONS request"

            parsed = urlparse(path)
            r = {
                'PATH_INFO':       self._get_path(parsed),
                'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'OPTIONS',
            }
            r.update(extra)
            return self.request(**r)

        def put(self, path, data={}, content_type=MULTIPART_CONTENT,
                **extra):
            "Construct a PUT request."

            put_data = self._encode_data(data, content_type)

            parsed = urlparse(path)
            r = {
                'CONTENT_LENGTH': len(put_data),
                'CONTENT_TYPE':   content_type,
                'PATH_INFO':      self._get_path(parsed),
                'QUERY_STRING':   parsed[4],
                'REQUEST_METHOD': 'PUT',
                'wsgi.input':     FakePayload(put_data),
            }
            r.update(extra)
            return self.request(**r)

        def delete(self, path, data={}, **extra):
            "Construct a DELETE request."

            parsed = urlparse(path)
            r = {
                'PATH_INFO':       self._get_path(parsed),
                'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'DELETE',
            }
            r.update(extra)
            return self.request(**r)



# coding: utf-8
from contextlib  import contextmanager
from django.conf import settings, UserSettingsHolder
from django.test import Client, TestCase, TransactionTestCase
from .signals    import setting_changed


__all__ = ("settings", "TransactionTestContext", "TestContext", "translation",
           "urlconf")


class TransactionTestContext(TransactionTestCase):
    # Avoid this class from being treated as a TestCase by Django's collector
    __name__ = 'blah_bada_boom'

    def __init__(self, fixtures=None, urls=None, client_class=None,
                 multi_db=None):
        if fixtures:
            self.fixtures = fixtures
        if urls:
            self.urls = urls
        if client_class:
            self.client_class = client_class
        if multi_db:
            self.multi_db = multi_db

    def __call__(self):
        """
        Wrapper around default __call__ method to perform common Django
        test set up. This means that user-defined Test Cases aren't required to
        include a call to super().setUp().
        """
        self._pre_setup()
        try:
            yield getattr(self, "client_class", Client)()
        finally:
            self._post_teardown()


class TestContext(TestCase, TransactionTestContext):
    """
    Does basically the same as TransactionTestContext, but surrounds every
    test with a transaction, monkey-patches the real transaction management
    routines to do nothing, and rollsback the test transaction at the end of
    the test. You have to use TransactionTestContext, if you need transaction
    management inside a test.
    """


@contextmanager
def settings(**kwargs):
    """
    Change one or more Django settings.
    """
    from django.conf import settings
    original = settings._wrapped
    try:
        settings._wrapped = UserSettingsHolder(settings._wrapped)
        sender = type(settings._wrapped)
        for key, value in kwargs.items():
            setattr(settings._wrapped, key, value)
            setting_changed.send(sender=sender, setting=key, value=value)
        yield
    finally:
        settings._wrapped = original
        sender = type(settings._wrapped)
        for key, value in kwargs.items():
            value = getattr(settings, key, None)
            setting_changed.send(sender=sender, setting=key, value=value)


@contextmanager
def translation(language_code, deactivate=False):
    """
    Port of django.utils.translation.override from Django 1.4

    @param language_code: a language code or ``None``. If ``None``, translation
                          is disabled and raw translation strings are used
    @param    deactivate: If ``True``, when leaving the manager revert to the
                          default behaviour (i.e. ``settings.LANGUAGE_CODE``)
                          rather than the translation that was active prior to
                          entering.
    """
    from django.utils import translation
    original = translation.get_language()
    if language_code is not None:
        translation.activate(language_code)
    else:
        translation.deactivate_all()
    try:
        yield
    finally:
        if deactivate:
            translation.deactivate()
        else:
            translation.activate(original)


@contextmanager
def urlconf(patterns):
    """
    A context manager that turns URL patterns into the global URLconf.

    This is useful when you have a local variable of URL patterns that

    :param patterns: list of `RegexURLPattern` or `RegexURLResolver` (what you
                     get from `patterns`)

    .. note:: Not thread safe
    """
    NOTSET = object()
    global urlpatterns
    # The extravagent effort here to preserve the current value of urlpatterns
    # is done to ensure nesting of `urlconf`.
    try:
        old = urlpatterns
    except NameError:
        old = NOTSET
    urlpatterns = patterns
    try:
        with settings(ROOT_URLCONF=__name__):
            yield
    finally:
        if old is NOTSET:
            del urlpatterns
        else:
            urlpatterns = old



# coding: utf-8
__all__ = ()


try:
    # Django 1.2 doesn't have this
    from django.utils import unittest
except ImportError:
    pass
else:
    from attest import capture_output, TestFailure, TestResult, get_reporter_by_name
    from django.test.simple import DjangoTestSuiteRunner


    __all__ += ("Runner", )


    class DummyProgress(object):
        start = update = finish = lambda *a, **k: None


    class TextResult(unittest.TextTestResult):
        def _exc_info_to_string(self, err, test):
            if err[0] is TestFailure:
                # retrieve stdout/stderr
                if self.buffer:
                    stdout = sys.stdout.getvalue().splitlines()
                    stderr = sys.stderr.getvalue().splitlines()
                else:
                    stdout, stderr = [], []

                # pull the test function out of the TestCase.
                test_func = getattr(test, test._testMethodName)
                result = TestResult(test=test_func, exc_info=err, time=0,
                                    error=err[1], stdout=stdout, stderr=stderr)
                reporter = get_reporter_by_name('auto')()
                reporter.begin(())
                reporter.progress = DummyProgress()
                reporter.failure(result)
                with capture_output() as (out, err):
                    try:
                        reporter.finished()
                    except SystemExit:
                        pass
                return '\n'.join(out + err)
            return super(TextResult, self)._exc_info_to_string(err, test)


    class TextRunner(unittest.TextTestRunner):
        resultclass = TextResult


    class Runner(DjangoTestSuiteRunner):
        def run_suite(self, suite, **kwargs):
            runner = TextRunner(verbosity=self.verbosity, failfast=self.failfast)
            return runner.run(suite)



# coding: utf-8
from django.core.urlresolvers import clear_url_caches
from django.dispatch import Signal

try:
    from django.test.signals import setting_changed
except ImportError:
    # Make our own signal that can be used any library that's interested and
    # wants to work in Django <1.4. Having this allows code to assume
    # `setting_changed` exists, making it simpler.
    setting_changed = Signal(providing_args=["setting", "value"])


def urlconf_caching(sender, setting, value, **kwargs):
    if setting == "ROOT_URLCONF":
        clear_url_caches()

setting_changed.connect(urlconf_caching)



# coding: utf-8
from .environment import setup
from attest import reporters
import sys


__all__ = ("auto_reporter", "AbstractReporter", "FancyReporter",
           "PlainReporter", "QuickFixReporter", "XmlReporter")


class ReporterMixin(object):
    """
    A mixin for reporters that handles setting up a Django environment.
    """
    def begin(self, tests):
        self.teardown = setup()
        return super(ReporterMixin, self).begin(tests)

    def finished(self):
        self.teardown()
        return super(ReporterMixin, self).finished()


def patched(reporter):
    return type(reporter.__name__,
                (ReporterMixin, reporter),
                {})


AbstractReporter = patched(reporters.AbstractReporter)
PlainReporter    = patched(reporters.PlainReporter)
FancyReporter    = patched(reporters.FancyReporter)
XmlReporter      = patched(reporters.XmlReporter)
QuickFixReporter = patched(reporters.QuickFixReporter)


def auto_reporter(**opts):
    # Let Attest make the decision, then return our own version. This only
    # works because we name our reporters the same as Attest
    suggested = reporters.auto_reporter(**opts)
    return getattr(sys.modules[__name__], type(suggested).__name__)

auto_reporter.test_loader = \
    lambda: reporters._test_loader_factory(auto_reporter)



# coding: utf8
from setuptools import setup


setup(
    name='django-attest',
    version='0.10.0',
    description='Provides Django specific testing helpers to Attest',

    author='Bradley Ayers',
    author_email='bradley.ayers@gmail.com',
    license='Simplified BSD',
    url='https://github.com/bradleyayers/django-attest/',

    packages=['django_attest'],
    install_requires=['Django >=1.2', 'Attest >=0.6dev', 'distribute', 'six'],
    use_2to3=True,

    entry_points={
        'attest.reporters': [
            'django-xml = django_attest:XmlReporter',
            'django-xunit = django_attest:XUnitReporter',
            'django-quickfix = django_attest:QuickFixReporter',
            'django-plain = django_attest:PlainReporter',
            'django-fancy = django_attest:FancyReporter',
            'django = django_attest:auto_reporter',
        ],
    },

    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# coding: utf-8
from attest import AssertImportHook, COMPILES_AST
from contextlib import contextmanager
import django
from django.test.simple import DjangoTestSuiteRunner
import logging
from pkg_resources import parse_version


__all__ = ("testing_environment", )


logger = logging.getLogger("django_attest")


if parse_version(django.get_version()) < parse_version('1.4'):
    logger.info("Django <1.4 has broken import infrastructure, Attest's"
                " assert hook will be disabled.")
    AssertImportHook.disable()
elif COMPILES_AST and not AssertImportHook.enabled:
    AssertImportHook.enable()


@contextmanager
def testing_environment():
    """
    Context manager to put Django into a state suitable for testing.
    """
    teardown = setup()
    try:
        yield
    finally:
        teardown()


def setup():
    """
    Setup the environment for Django (create databases, turn on DEBUG, etc).

    :returns: teardown function
    """
    # Use Django's test suite runner, as it sets up test databases nicely
    runner = DjangoTestSuiteRunner()
    runner.setup_test_environment()
    old_config = runner.setup_databases()

    def teardown():
        runner.teardown_databases(old_config)
        runner.teardown_test_environment()

    return teardown
